@@ -16,10 +16,12 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <linux/input.h>
 #include <sys/ioctl.h>
+#include <sys/types.h>
 #include <dirent.h>
 #include <argp.h>
 
@@ -28,8 +30,23 @@
 /* Default place where the keymaps will be stored */
 #define CFGDIR "/etc/rc_keymaps"
 
+struct input_keymap_entry_v2 {
+#define KEYMAP_BY_INDEX	(1 << 0)
+	u_int8_t  flags;
+	u_int8_t  len;
+	u_int16_t index;
+	u_int32_t keycode;
+	u_int8_t  scancode[32];
+};
+
+#ifndef EVIOCGKEYCODE_V2
+#define EVIOCGKEYCODE_V2	_IOR('E', 0x04, struct input_keymap_entry_v2)
+#define EVIOCSKEYCODE_V2	_IOW('E', 0x04, struct input_keymap_entry_v2)
+#endif
+
 struct keytable {
-	int codes[2];
+	u_int32_t codes[2];
+	struct input_keymap_entry_v2 keymap;
 	struct keytable *next;
 };
 
@@ -83,7 +100,7 @@ static int parse_code(char *string)
 	return -1;
 }
 
-const char *argp_program_version = "IR keytable control version 0.1.0";
+const char *argp_program_version = "IR keytable control version "V4L_UTILS_VERSION;
 const char *argp_program_bug_address = "Mauro Carvalho Chehab <mchehab@redhat.com>";
 
 static const char doc[] = "\nAllows get/set IR keycode/scancode tables\n"
@@ -91,9 +108,11 @@ static const char doc[] = "\nAllows get/set IR keycode/scancode tables\n"
 	"\nOn the options bellow, the arguments are:\n"
 	"  DEV      - the /dev/input/event* device to control\n"
 	"  SYSDEV   - the ir class as found at /sys/class/rc\n"
-	"  TABLE    - a file wit a set of scancode=keycode value pairs\n"
+	"  TABLE    - a file with a set of scancode=keycode value pairs\n"
 	"  SCANKEY  - a set of scancode1=keycode1,scancode2=keycode2.. value pairs\n"
 	"  PROTOCOL - protocol name (nec, rc-5, rc-6, other) to be enabled\n"
+	"  DELAY    - Delay before repeating a keystroke\n"
+	"  PERIOD   - Period to repeat a keystroke\n"
 	"  CFGFILE  - configuration file that associates a driver/table name with a keymap file\n"
 	"\nOptions can be combined together.";
 
@@ -101,11 +120,14 @@ static const struct argp_option options[] = {
 	{"verbose",	'v',	0,		0,	"enables debug messages", 0},
 	{"clear",	'c',	0,		0,	"clears the old table", 0},
 	{"sysdev",	's',	"SYSDEV",	0,	"ir class device to control", 0},
+	{"test",	't',	0,		0,	"test if IR is generating events", 0},
 	{"device",	'd',	"DEV",		0,	"ir device to control", 0},
 	{"read",	'r',	0,		0,	"reads the current scancode/keycode table", 0},
 	{"write",	'w',	"TABLE",	0,	"write (adds) the scancodes to the device scancode/keycode table from an specified file", 0},
 	{"set-key",	'k',	"SCANKEY",	0,	"Change scan/key pairs", 0},
 	{"protocol",	'p',	"PROTOCOL",	0,	"Protocol to enable (the other ones will be disabled). To enable more than one, use the option more than one time", 0},
+	{"delay",	'D',	"DELAY",	0,	"Sets the delay before repeating a keystroke", 0},
+	{"period",	'P',	"PERIOD",	0,	"Sets the period to repeat a keystroke", 0},
 	{"auto-load",	'a',	"CFGFILE",	0,	"Auto-load a table, based on a configuration file. Only works with sysdev.", 0},
 	{ 0, 0, 0, 0, 0, 0 }
 };
@@ -118,13 +140,17 @@ static const char args_doc[] =
 /* Static vars to store the parameters */
 static char *devclass = "rc0";
 static char *devname = NULL;
-static int read = 0;
+static int readtable = 0;
 static int clear = 0;
 static int debug = 0;
+static int test = 0;
+static int delay = 0;
+static int period = 0;
 static enum ir_protocols ch_proto = 0;
 
 struct keytable keys = {
-	{0, 0}, NULL
+	.codes = {0, 0},
+	.next = NULL
 };
 
 
@@ -132,6 +158,11 @@ struct cfgfile cfg = {
 	NULL, NULL, NULL, NULL
 };
 
+/*
+ * Stores the input layer protocol version
+ */
+static int input_protocol_version = 0;
+
 /*
  * Values that are read only via sysfs node
  */
@@ -236,7 +267,7 @@ static error_t parse_keyfile(char *fname, char **table)
 				perror("value");
 		}
 
-		nextkey->codes[0] = (unsigned) strtol(scancode, NULL, 0);
+		nextkey->codes[0] = (unsigned) strtoul(scancode, NULL, 0);
 		nextkey->codes[1] = (unsigned) value;
 		nextkey->next = calloc(1, sizeof(*nextkey));
 		if (!nextkey->next) {
@@ -337,9 +368,18 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 	case 'v':
 		debug++;
 		break;
+	case 't':
+		test++;
+		break;
 	case 'c':
 		clear++;
 		break;
+	case 'D':
+		delay = atoi(arg);
+		break;
+	case 'P':
+		period = atoi(arg);
+		break;
 	case 'd':
 		devname = arg;
 		break;
@@ -347,7 +387,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		devclass = arg;
 		break;
 	case 'r':
-		read++;
+		readtable++;
 		break;
 	case 'w': {
 		char *name = NULL;
@@ -370,7 +410,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		do {
 			if (!p)
 				goto err_inval;
-			nextkey->codes[0] = strtol(p, NULL, 0);
+			nextkey->codes[0] = strtoul(p, NULL, 0);
 			if (errno)
 				goto err_inval;
 
@@ -386,7 +426,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			nextkey->codes[1] = key;
 
 			if (debug)
-				fprintf(stderr, "scancode %i=%i\n",
+				fprintf(stderr, "scancode %u=%u\n",
 					nextkey->codes[0], nextkey->codes[1]);
 
 			nextkey->next = calloc(1, sizeof(keys));
@@ -1050,7 +1090,7 @@ static int get_attribs(struct rc_device *rc_dev, char *sysfs_name)
 
 static int set_proto(struct rc_device *rc_dev)
 {
-	int rc;
+	int rc = 0;
 
 	if (rc_dev->version == VERSION_2) {
 		rc = v2_set_protocols(rc_dev);
@@ -1080,18 +1120,48 @@ static int set_proto(struct rc_device *rc_dev)
 	return rc;
 }
 
+static int get_input_protocol_version(int fd)
+{
+	if (ioctl(fd, EVIOCGVERSION, &input_protocol_version) < 0) {
+		fprintf(stderr,
+			"Unable to query evdev protocol version: %s\n",
+			strerror(errno));
+		return errno;
+	}
+	if (debug)
+		fprintf(stderr, "Input Protocol version: 0x%08x\n",
+			input_protocol_version);
+
+	return 0;
+}
+
 static void clear_table(int fd)
 {
 	int i, j;
-	int codes[2];
+	u_int32_t codes[2];
+	struct input_keymap_entry_v2 entry;
 
 	/* Clears old table */
-	for (j = 0; j < 256; j++) {
-		for (i = 0; i < 256; i++) {
-			codes[0] = (j << 8) | i;
-			codes[1] = KEY_RESERVED;
-			ioctl(fd, EVIOCSKEYCODE, codes);
+	if (input_protocol_version < 0x10001) {
+		for (j = 0; j < 256; j++) {
+			for (i = 0; i < 256; i++) {
+				codes[0] = (j << 8) | i;
+				codes[1] = KEY_RESERVED;
+				ioctl(fd, EVIOCSKEYCODE, codes);
+			}
 		}
+	} else {
+		memset(&entry, '\0', sizeof(entry));
+		i = 0;
+		do {
+			entry.flags = KEYMAP_BY_INDEX;
+			entry.keycode = KEY_RESERVED;
+			entry.index = 0;
+
+			i++;
+			if (debug)
+				fprintf(stderr, "Deleting entry %d\n", i);
+		} while (ioctl(fd, EVIOCSKEYCODE_V2, &entry) == 0);
 	}
 }
 
@@ -1133,7 +1203,67 @@ static void display_proto(struct rc_device *rc_dev)
 	fprintf(stderr, "\n");
 }
 
-static void display_table(struct rc_device *rc_dev, int fd)
+static void test_event(int fd)
+{
+	struct input_event ev[64];
+	int rd, i;
+
+	printf ("Testing events. Please, press CTRL-C to abort.\n");
+	while (1) {
+		rd = read(fd, ev, sizeof(ev));
+
+		if (rd < (int) sizeof(struct input_event)) {
+			perror("Error reading event");
+			return;
+		}
+
+		for (i = 0; i < rd / sizeof(struct input_event); i++) {
+			switch (ev[i].type) {
+			case EV_MSC:
+				if (ev[i].code != MSC_SCAN)
+					break;
+				printf("%ld.%06ld: event MSC: scancode = %02x\n",
+					ev[i].time.tv_sec, ev[i].time.tv_usec, ev[i].value);
+				break;
+			case EV_KEY: 			{
+				struct parse_key *p;
+				char *name = "";
+
+				printf("%ld.%06ld: event key %s: ",
+					ev[i].time.tv_sec, ev[i].time.tv_usec,
+					(ev[i].value == 0) ? "up" : "down"
+					);
+
+				for (p = keynames; p->name != NULL; p++) {
+					if (p->value == ev[i].code) {
+						name = p->name;
+						break;
+					}
+				}
+				printf("%s (0x%04x)\n", name, ev[i].code);
+
+				break;
+			}
+			case EV_REP:
+				printf("%ld.%06ld: event repeat: %d\n",
+					ev[i].time.tv_sec, ev[i].time.tv_usec,
+					ev[i].value);
+				break;
+			case EV_SYN:
+				printf("%ld.%06ld: event sync\n",
+					ev[i].time.tv_sec, ev[i].time.tv_usec);
+				break;
+			default:
+				printf("%ld.%06ld: event type %d: value: %d\n",
+					ev[i].time.tv_sec, ev[i].time.tv_usec,
+					ev[i].type, ev[i].value);
+				break;
+			}
+		}
+	}
+}
+
+static void display_table_v1(struct rc_device *rc_dev, int fd)
 {
 	unsigned int i, j;
 
@@ -1142,13 +1272,118 @@ static void display_table(struct rc_device *rc_dev, int fd)
 			int codes[2];
 
 			codes[0] = (j << 8) | i;
-			if (!ioctl(fd, EVIOCGKEYCODE, codes) && codes[1] != KEY_RESERVED)
+			if (ioctl(fd, EVIOCGKEYCODE, codes) == -1)
+				perror("EVIOCGKEYCODE");
+			else if (codes[1] != KEY_RESERVED)
 				prtcode(codes);
 		}
 	}
 	display_proto(rc_dev);
 }
 
+static void display_table_v2(struct rc_device *rc_dev, int fd)
+{
+	int i;
+	struct input_keymap_entry_v2 entry;
+	int codes[2];
+
+	memset(&entry, '\0', sizeof(entry));
+	i = 0;
+	do {
+		entry.flags = KEYMAP_BY_INDEX;
+		entry.index = i;
+		entry.len = sizeof(u_int32_t);
+
+		if (ioctl(fd, EVIOCGKEYCODE_V2, &entry) == -1)
+			break;
+
+		/* FIXME: Extend it to support scancodes > 32 bits */
+		codes[0] = ((u_int32_t *)entry.scancode)[0];
+		codes[1] = entry.keycode;
+
+		prtcode(codes);
+		i++;
+	} while (1);
+	display_proto(rc_dev);
+}
+
+static void display_table(struct rc_device *rc_dev, int fd)
+{
+	if (input_protocol_version < 0x10001)
+		display_table_v1(rc_dev, fd);
+	else
+		display_table_v2(rc_dev, fd);
+}
+
+static int set_rate(int fd, unsigned int delay, unsigned int period)
+{
+	unsigned int rep[2] = { delay, period };
+
+	if (ioctl(fd, EVIOCSREP, rep) < 0) {
+		perror("evdev ioctl");
+		return -1;
+	}
+
+	printf("Changed Repeat delay to %d ms and repeat period to %d ms\n", delay, period);
+	return 0;
+}
+
+static int get_rate(int fd, unsigned int *delay, unsigned int *period)
+{
+	unsigned int rep[2];
+
+	if (ioctl(fd, EVIOCGREP, rep) < 0) {
+		perror("evdev ioctl");
+		return -1;
+	}
+	*delay = rep[0];
+	*period = rep[1];
+	printf("Repeat delay = %d ms, repeat period = %d ms\n", *delay, *period);
+	return 0;
+}
+
+static void show_evdev_attribs(int fd)
+{
+	unsigned int delay, period;
+
+	printf("\t");
+	get_rate(fd, &delay, &period);
+}
+
+static int show_sysfs_attribs(struct rc_device *rc_dev)
+{
+	static struct sysfs_names *names, *cur;
+	int fd;
+
+	names = find_device(NULL);
+	if (!names)
+		return -1;
+	for (cur = names; cur->next; cur = cur->next) {
+		if (cur->name) {
+			if (get_attribs(rc_dev, cur->name))
+				return -1;
+			fprintf(stderr, "Found %s (%s) with:\n",
+				rc_dev->sysfs_name,
+				rc_dev->input_name);
+			fprintf(stderr, "\tDriver %s, table %s\n",
+				rc_dev->drv_name,
+				rc_dev->keytable_name);
+			fprintf(stderr, "\tSupported protocols: ");
+			show_proto(rc_dev->supported);
+			fprintf(stderr, "\n\t");
+			display_proto(rc_dev);
+			fd = open(rc_dev->input_name, O_RDONLY);
+			if (fd > 0) {
+				show_evdev_attribs(fd);
+				close(fd);
+			} else {
+				printf("\tExtra capabilities: <access denied>\n");
+			}
+		}
+	}
+	return 0;
+}
+
 int main(int argc, char *argv[])
 {
 	int dev_from_class = 0, write_cnt;
@@ -1159,28 +1394,10 @@ int main(int argc, char *argv[])
 	argp_parse(&argp, argc, argv, 0, 0, 0);
 
 	/* Just list all devices */
-	if (!clear && !read && !keys.next && !ch_proto && !cfg.next) {
-		static struct sysfs_names *names, *cur;
-
-		names = find_device(NULL);
-		if (!names)
+	if (!clear && !readtable && !keys.next && !ch_proto && !cfg.next && !test && !delay && !period) {
+		if (show_sysfs_attribs(&rc_dev))
 			return -1;
-		for (cur = names; cur->next; cur = cur->next) {
-			if (cur->name) {
-				if (get_attribs(&rc_dev, cur->name))
-					return -1;
-				fprintf(stderr, "Found %s (%s) with:\n",
-					rc_dev.sysfs_name,
-					rc_dev.input_name);
-				fprintf(stderr, "\tDriver %s, table %s\n",
-					rc_dev.drv_name,
-					rc_dev.keytable_name);
-				fprintf(stderr, "\tSupported protocols: ");
-				show_proto(rc_dev.supported);
-				fprintf(stderr, "\t");
-				display_proto(&rc_dev);
-			}
-		}
+
 		return 0;
 	}
 
@@ -1210,9 +1427,9 @@ int main(int argc, char *argv[])
 		int rc;
 
 		for (cur = &cfg; cur->next; cur = cur->next) {
-			if (strcasecmp(cur->driver, rc_dev.drv_name) && strcasecmp(cur->driver, "*"))
+			if ((!rc_dev.drv_name || strcasecmp(cur->driver, rc_dev.drv_name)) && strcasecmp(cur->driver, "*"))
 				continue;
-			if (strcasecmp(cur->table, rc_dev.keytable_name) && strcasecmp(cur->table, "*"))
+			if ((!rc_dev.keytable_name || strcasecmp(cur->table, rc_dev.keytable_name)) && strcasecmp(cur->table, "*"))
 				continue;
 			break;
 		}
@@ -1252,6 +1469,8 @@ int main(int argc, char *argv[])
 	}
 	if (dev_from_class)
 		free(devname);
+	if (get_input_protocol_version(fd))
+		return -1;
 
 	/*
 	 * First step: clear, if --clear is specified
@@ -1285,8 +1504,24 @@ int main(int argc, char *argv[])
 	/*
 	 * Fourth step: display current keytable
 	 */
-	if (read)
+	if (readtable)
 		display_table(&rc_dev, fd);
 
+	/*
+	 * Fiveth step: change repeat rate/delay
+	 */
+	if (delay || period) {
+		unsigned int new_delay, new_period;
+		get_rate(fd, &new_delay, &new_period);
+		if (delay)
+			new_delay = delay;
+		if (period)
+			new_period = period;
+		set_rate(fd, new_delay, new_period);
+	}
+
+	if (test)
+		test_event(fd);
+
 	return 0;
 }
