@@ -23,6 +23,7 @@
 
 #include <string>
 #include <list>
+#include <set>
 #include <linux/videodev2.h>
 #include <libv4l2.h>
 
@@ -37,6 +38,7 @@ struct test_queryctrl: v4l2_queryctrl {
 };
 
 typedef std::list<test_queryctrl> qctrl_list;
+typedef std::set<__u32> pixfmt_set;
 
 struct node {
 	int fd;
@@ -55,6 +57,8 @@ struct node {
 	unsigned std_controls;
 	unsigned priv_controls;
 	qctrl_list controls;
+	__u32 fbuf_caps;
+	pixfmt_set buftype_pixfmts[V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE + 1];
 };
 
 #define info(fmt, args...) 					\
@@ -70,12 +74,18 @@ struct node {
  			printf("\t\twarn: " fmt, ##args);	\
 	} while (0)
 
-#define fail(fmt, args...) 			\
-({ 						\
- 	printf("\t\tfail: " fmt, ##args);	\
-	1;					\
+#define fail(fmt, args...) 						\
+({ 									\
+ 	printf("\t\tfail: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
+	1;								\
 })
 
+#define fail_on_test(test) 				\
+	do {						\
+	 	if (test)				\
+			return fail("%s\n", #test);	\
+	} while (0)
+
 static inline int test_open(const char *file, int oflag)
 {
  	return wrapper ? v4l2_open(file, oflag) : open(file, oflag);
@@ -91,10 +101,28 @@ static inline int test_ioctl(int fd, int cmd, void *arg)
 	return wrapper ? v4l2_ioctl(fd, cmd, arg) : ioctl(fd, cmd, arg);
 }
 
+static inline int check_fract(const struct v4l2_fract *f)
+{
+	if (f->numerator && f->denominator)
+		return 0;
+	return 1;
+}
+
+static inline double fract2f(const struct v4l2_fract *f)
+{
+	return (double)f->numerator / (double)f->denominator;
+}
+
 int doioctl_name(struct node *node, unsigned long int request, void *parm, const char *name);
 #define doioctl(n, r, p) doioctl_name(n, r, p, #r)
 
 std::string cap2s(unsigned cap);
+std::string buftype2s(int type);
+static inline std::string buftype2s(enum v4l2_buf_type type)
+{
+       return buftype2s((int)type);
+}
+
 const char *ok(int res);
 int check_string(const char *s, size_t len);
 int check_ustring(const __u8 *s, int len);
@@ -129,4 +157,10 @@ int testStd(struct node *node);
 int testPresets(struct node *node);
 int testCustomTimings(struct node *node);
 
+// Format ioctl tests
+int testEnumFormats(struct node *node);
+int testFBuf(struct node *node);
+int testFormats(struct node *node);
+int testSlicedVBICap(struct node *node);
+
 #endif
